/*!
   \file domain_algorithm_redistribute.cpp
   \brief Algorithm for redistribute a domain.
 */
#include "domain_algorithm_redistribute.hpp"
#include "redistribute_domain.hpp"
#include "domain.hpp"
#include "grid.hpp"
#include "grid_transformation_factory_impl.hpp"

namespace xios
{

  shared_ptr<CGenericAlgorithmTransformation> CDomainAlgorithmRedistribute::create(bool isSource, CGrid* gridDst, CGrid* gridSrc,
                                                               CTransformation<CDomain>* transformation,
                                                               int elementPositionInGrid,
                                                               std::map<int, int>& elementPositionInGridSrc2ScalarPosition,
                                                               std::map<int, int>& elementPositionInGridSrc2AxisPosition,
                                                               std::map<int, int>& elementPositionInGridSrc2DomainPosition, 
                                                               std::map<int, int>& elementPositionInGridDst2ScalarPosition,
                                                               std::map<int, int>& elementPositionInGridDst2AxisPosition,
                                                               std::map<int, int>& elementPositionInGridDst2DomainPosition)
  TRY
  {
    std::vector<CDomain*> domainListDestP = gridDst->getDomains();
    std::vector<CDomain*> domainListSrcP  = gridSrc->getDomains();

    CRedistributeDomain* redistributeDomain = dynamic_cast<CRedistributeDomain*> (transformation);
    int domainDstIndex = elementPositionInGridDst2DomainPosition[elementPositionInGrid];
    int domainSrcIndex = elementPositionInGridSrc2DomainPosition[elementPositionInGrid];

    return make_shared<CDomainAlgorithmRedistribute>(isSource, domainListDestP[domainDstIndex], domainListSrcP[domainSrcIndex], redistributeDomain);
  }
  CATCH

  bool CDomainAlgorithmRedistribute::dummyRegistered_ = CDomainAlgorithmRedistribute::registerTrans();
  bool CDomainAlgorithmRedistribute::registerTrans()
  TRY
  {
    return CGridTransformationFactory<CDomain>::registerTransformation(TRANS_REDISTRIBUTE_DOMAIN, create);
  }
  CATCH

  CDomainAlgorithmRedistribute::CDomainAlgorithmRedistribute(bool isSource, CDomain* domainDestination, CDomain* domainSource, CRedistributeDomain* redistributeDomain)
  : CAlgorithmTransformationTransfer(isSource)
  TRY
  {


    domainDestination->checkAttributes() ;
  }
  CATCH


}
